package com.hungraim.ltc.service.impl;

import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.dao.*;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.account.ChDisabDetailAccounts;
import com.hungraim.ltc.pojo.entity.disable.ChCfgSrvEmpType;
import com.hungraim.ltc.pojo.entity.disable.ChSrvOrgan;
import com.hungraim.ltc.pojo.entity.riskLevel.ChSrvempRiskLevel;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvOrganEmp;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTaskDetail;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTaskDetailDelete;
import com.hungraim.ltc.pojo.vo.disable.ChSrvTaskDetail2Vo;
import com.hungraim.ltc.pojo.vo.disable.SrvOrganEmpExportVo;
import com.hungraim.ltc.pojo.vo.disable.SrvOrganEmpListVo;
import com.hungraim.ltc.pojo.vo.disable.SrvOrganEmpVo;
import com.hungraim.ltc.pojo.vo.system.AttachUpdateVo;
import com.hungraim.ltc.service.OrganService;
import com.hungraim.ltc.service.SrvOrganEmpService;
import com.hungraim.ltc.util.AttachType;
import com.hungraim.ltc.util.DateUtils;
import com.hungraim.ltc.util.Result;
import com.hungraim.ltc.util.ResultCode;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
@Slf4j
public class SrvOrganEmpServiceImpl implements SrvOrganEmpService {

    private final ChSrvOrganEmpMapper chSrvOrganEmpMapper;

    private final OrganService organService;

    private final AttachFeignService attachFeignService;

    private final SrvEmpTypeMapper srvEmpTypeMapper;

    private final SrvOrganMapper srvOrganMapper;

    @Autowired
    private ChSrvempRiskLevelMapper chSrvempRiskLevelMapper;

    private final ChSrvTaskDetailMapper chSrvTaskDetailMapper;

    private final ChSrvTaskDetailDeleteMapper chSrvTaskDetailDeleteMapper;



    @Override
    public Page<SrvOrganEmpListVo> getSrvOrganEmpListVo(Page<SrvOrganEmpListVo> page, Long organId, Boolean isInvolveChild, String empName, String srvOrganName, String certifNum, Short isLock, Short empType,Long srvOrganId) {
        List<Long> organIds = organService.getOrganIds(organId, isInvolveChild);
        return chSrvOrganEmpMapper.getSrvOrganEmpListVo(page, organIds, empName, srvOrganName, certifNum, isLock, empType,srvOrganId);
    }



    @Override
    public int updateSrvEmpStatus(Long empId, Short isLock,String certifNum) {
        int queryResult = 0;
        //修改成有效状态前检查是否存在有效数据。存在则不允许修改
        if(isLock==0){
            QueryWrapper<ChSrvOrganEmp> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("CERTIF_NUM", certifNum);
            queryWrapper.eq("IS_LOCK",0);
            queryResult = chSrvOrganEmpMapper.selectCount(queryWrapper);
            if(queryResult>0){
                return 0;
            }
        }
        ChSrvOrganEmp chSrvOrganEmp = new ChSrvOrganEmp();
        chSrvOrganEmp.setEmpId(empId);
        chSrvOrganEmp.setIsLock(isLock);
        queryResult = chSrvOrganEmpMapper.updateById(chSrvOrganEmp);
        return queryResult;
    }

    @Override
    public SrvOrganEmpVo getSrvOrganEmp(Long empId) {
        SrvOrganEmpVo srvOrganEmpVo = new SrvOrganEmpVo();
        ChSrvOrganEmp chSrvOrganEmp = chSrvOrganEmpMapper.selectById(empId);
        Long empTypeId = chSrvOrganEmp.getEmpTypeId();
        if(empTypeId!=null){
            ChCfgSrvEmpType chCfgSrvEmpType = srvEmpTypeMapper.selectById(empTypeId);
            srvOrganEmpVo.setEmpCateId(chCfgSrvEmpType.getEmpCateId());
        }else{
            srvOrganEmpVo.setEmpCateId(0);
        }
        Long srvOrganId = chSrvOrganEmp.getSrvOrganId();
        ChSrvOrgan chSrvOrgan = srvOrganMapper.selectById(srvOrganId);
        BeanUtils.copyProperties(chSrvOrganEmp, srvOrganEmpVo);
        srvOrganEmpVo.setUnifiedSocialCreditCode(chSrvOrgan.getUnifiedSocialCreditCode());
        try {
            Result<List<ChFndAttach>> listResult = attachFeignService.selectAttachs(empId, AttachType.SRV_ORGAN_EMP.getUploadType());
            if (ResultCode.SUCCESS.getCode().equals(listResult.getCode())) {
                srvOrganEmpVo.setFileDataList(listResult.getData());
            } else {
                log.error("请求获取护理人员上传文件出错，出错信息：{}", listResult.getMsg());
            }
        } catch (Exception e) {
            log.error("请求获取护理人员上传文件出错，出错信息：{}", e.getMessage());
        }
        return srvOrganEmpVo;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int saveOrUpdateSrvOrganEmp(SrvOrganEmpVo srvOrganEmpVo) {
        Long empId = srvOrganEmpVo.getEmpId();
        ChSrvOrganEmp chSrvOrganEmp = new ChSrvOrganEmp();
        BeanUtils.copyProperties(srvOrganEmpVo, chSrvOrganEmp);
        int queryResult = 0;
        if (null == empId) {
            QueryWrapper<ChSrvOrganEmp> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("CERTIF_NUM", chSrvOrganEmp.getCertifNum());
            queryWrapper.eq("IS_LOCK",0);
            queryResult = chSrvOrganEmpMapper.selectCount(queryWrapper);
            if(queryResult>0){
               return queryResult;
            }
            // 新增
            chSrvOrganEmp.setEffTime(new Date());
            chSrvOrganEmp.setIsLock((short) 0);
            chSrvOrganEmp.setEmpPwd("ch123456");
            chSrvOrganEmpMapper.insert(chSrvOrganEmp);
            updateAttachs(srvOrganEmpVo, chSrvOrganEmp.getEmpId());

            ChSrvempRiskLevel chSrvempRiskLevel = new ChSrvempRiskLevel();
            chSrvempRiskLevel.setRiskLevel(0L);
            chSrvempRiskLevel.setSrvEmpId(Long.valueOf(chSrvOrganEmp.getEmpId()));
            chSrvempRiskLevel.setStatus(0);
            chSrvempRiskLevel.setFcd(new Date());
            chSrvempRiskLevelMapper.insert(chSrvempRiskLevel);
        } else {
            // 修改
            chSrvOrganEmpMapper.updateById(chSrvOrganEmp);
            updateAttachs(srvOrganEmpVo, empId);
        }
        return queryResult;
    }

    // 抽取方法
    private void updateAttachs(SrvOrganEmpVo srvOrganEmpVo, Long empId) {
        AttachUpdateVo attachUpdateVo = new AttachUpdateVo();
        attachUpdateVo.setChFndAttachs(srvOrganEmpVo.getFileDataList());
        attachUpdateVo.setCtrlId(empId);
        attachUpdateVo.setAttachType(AttachType.SRV_ORGAN_EMP.getUploadType());
        attachFeignService.updateAttachs(attachUpdateVo);
    }


    @Override
    public Workbook srvOrganExport(Long organId,Long srvOrganId, Boolean isInvolveChild, String empName, String srvOrganName, String certifNum,
                                   Short isLock, Short empType) {
        List<Long> organIds = organService.getOrganIds(organId, isInvolveChild);
        List<SrvOrganEmpExportVo> srvOrganEmpExportList = chSrvOrganEmpMapper.getSrvOrganEmpExportList(organIds,srvOrganId, empName, srvOrganName, certifNum, isLock, empType);
        ExportParams exportParams = new ExportParams();
        exportParams.setType(ExcelType.XSSF); // 对应xlsx
        Workbook workbook = ExcelExportUtil.exportExcel(exportParams, SrvOrganEmpExportVo.class, srvOrganEmpExportList);
        return workbook;
    }

    @Override
    public Page<ChSrvTaskDetail2Vo> chSrvTaskDetailQuery(Page<ChSrvTaskDetail2Vo> page, Long empId, String empName,String status,String sevrTime) {
        Page<ChSrvTaskDetail2Vo> chSrvTaskDetailVoPage = chSrvTaskDetailMapper.chSrvTaskDetailQuery(page, empId,empName,status,sevrTime);
        chSrvTaskDetailVoPage.getRecords().stream().filter(chSrvTaskDetail -> {
            if (null != chSrvTaskDetail.getSevStarTime()){
                chSrvTaskDetail.setSevStarTime(chSrvTaskDetail.getSevStarTime().substring(0,19));
            }
            if (null != chSrvTaskDetail.getSevEndTime()){
                chSrvTaskDetail.setSevEndTime(chSrvTaskDetail.getSevEndTime().substring(0,19));
            }
            return true;
        }).collect(Collectors.toList());
        return chSrvTaskDetailVoPage;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Result chSrvTaskDetailDelete(long taskDetailId,String userName) {
        ChSrvTaskDetail chSrvTaskDetail = chSrvTaskDetailMapper.selectById(taskDetailId);
        ChSrvTaskDetailDelete chSrvTaskDetailDelete = new ChSrvTaskDetailDelete();
        BeanUtils.copyProperties(chSrvTaskDetail,chSrvTaskDetailDelete);
        chSrvTaskDetailDelete.setDeleteDate(new Date());
        chSrvTaskDetailDelete.setDeleteName(userName);
        chSrvTaskDetailDeleteMapper.insert(chSrvTaskDetailDelete);
        if( 4 == chSrvTaskDetail.getStatus()) {
            // 开始时间
            Date srvStartTime = chSrvTaskDetail.getSrvStartTime();
            String srvStartTimeStr = DateUtils.dateTimeToStr(srvStartTime);
            // 结束时间
            Date srvEndTime = chSrvTaskDetail.getSrvEndTime();
            String srvEndTimeStr = DateUtils.dateTimeToStr(srvEndTime);
            Long min = dateDiff(srvStartTimeStr, srvEndTimeStr, "yyyy-MM-dd HH:mm:ss", "min");
            if(min>=90) {
                Date srvDate = chSrvTaskDetail.getSrvDate();
                String srvDateStr = DateUtils.dateToStrYm(srvDate);
                Long srvTaskId = chSrvTaskDetail.getSrvTaskId();
                String disabInfoId = chSrvTaskDetailMapper.selectBySrvTaskId(String.valueOf(srvTaskId));
                ChDisabDetailAccounts chDisabDetailAccounts = chSrvTaskDetailMapper.selectAccountsDetail(disabInfoId, srvDateStr);
                if (Objects.nonNull(chDisabDetailAccounts)) {
                    Short status = chDisabDetailAccounts.getStatus();
                    if (status != 0) {
                        // 已申报不能进行删除
                        return Result.failed("已申报不能进行删除");
                    }
                    BigDecimal accountsAllCost = chDisabDetailAccounts.getAccountsAllCost();
                    BigDecimal accountsPersonalCost = chDisabDetailAccounts.getAccountsPersonalCost();
                    BigDecimal accountsOverallCost = chDisabDetailAccounts.getAccountsOverallCost();
                    BigDecimal accountsAllCostsub = accountsAllCost.subtract(new BigDecimal(75));
                    BigDecimal accountsOverallCostsub = accountsOverallCost.subtract(new BigDecimal(60));
                    BigDecimal accountsPersonalCostsub = accountsPersonalCost.subtract(new BigDecimal(15));
                    Long accountsDetailId = chDisabDetailAccounts.getAccountsDetailId();
                    chSrvTaskDetailMapper.updateChDisabDetailAccounts(accountsDetailId, accountsAllCostsub, accountsOverallCostsub, accountsPersonalCostsub);
                }
            }
        }
        chSrvTaskDetailMapper.deleteFinshItem(taskDetailId);
        int srv = chSrvTaskDetailMapper.updateByTaskDetailId(String.valueOf(chSrvTaskDetail.getTaskDetailId()));
        if(srv>0){
            return Result.success();
        }else {
            return Result.failed("删除失败");
        }
    }

    @Override
    public Workbook chSrvTaskDetailExport(Long empId, String empName, String status, String sevrTime) {
        List<ChSrvTaskDetail2Vo> chSrvTaskDetailExportList = chSrvTaskDetailMapper.chSrvTaskDetailExportList(empId,empName,status,sevrTime);
        ExportParams exportParams = new ExportParams();
        exportParams.setType(ExcelType.XSSF); // 对应xlsx
        Workbook workbook = ExcelExportUtil.exportExcel(exportParams, ChSrvTaskDetail2Vo.class, chSrvTaskDetailExportList);
        return workbook;
    }

    /**
     * 计算2个时间相差的天数、小时、分钟、秒
     * @param startTime 开始时间
     * @param endTime 截止时间
     * @param format 时间格式 yyyy-MM-dd HH:mm:ss
     * @param str 返回的数据为：day-天、hour-小时、min-分钟、second-秒
     * @return
     */
    public static Long dateDiff(String startTime, String endTime,  String format, String str) {
        // 按照传入的格式生成一个simpledateformate对象
        SimpleDateFormat sd = new SimpleDateFormat(format);
        long nd = 1000 * 24 * 60 * 60;// 一天的毫秒数
        long nh = 1000 * 60 * 60;// 一小时的毫秒数
        long nm = 1000 * 60;// 一分钟的毫秒数
        long ns = 1000;// 一秒钟的毫秒数
        long diff;
        long day = 0;
        long hour = 0;
        long min = 0;
        long second = 0;
        // 获得两个时间的毫秒时间差异
        try {
            diff = sd.parse(endTime).getTime() - sd.parse(startTime).getTime();
            // 计算差多少天
            day = diff / nd;
            // 计算差多少小时
            hour = diff / nh ;
            // 计算差多少分钟
            min = diff / nm ;
            // 计算差多少秒
            second = diff / ns;
            // 输出结果
            System.out.println("时间相差：" + day + "天" +
                    (hour - day * 24) + "小时"
                    + (min - day * 24 * 60) + "分钟" +
                    second + "秒。");
            /*System.out.println("hour=" + hour + ",min=" + min);*/
        } catch (ParseException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        if (str.equalsIgnoreCase("day")) {
            return day;
        } else if(str.equalsIgnoreCase("hour")) {
            return hour;
        } else if(str.equalsIgnoreCase("min")) {
            return min;
        } else {
            return second;
        }
    }



    @Override
    public Result selectEmpBySrvOrganId(String srvOrganId){
        LambdaQueryWrapper lambdaQueryWrapper =   new LambdaQueryWrapper<ChSrvOrganEmp>()
        .eq(ChSrvOrganEmp::getSrvOrganId,Long.valueOf(srvOrganId))
        .eq(ChSrvOrganEmp::getIsLock,(short)0);
        List<ChSrvOrganEmp> list = chSrvOrganEmpMapper.selectList(lambdaQueryWrapper);
        return Result.success(list);

    }



}
