package com.hungraim.ltc.governance.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.hungraim.ltc.api.AccountFeignService;
import com.hungraim.ltc.api.AttachFeignService;
import com.hungraim.ltc.governance.dao.*;
import com.hungraim.ltc.governance.service.SrvSendService;
import com.hungraim.ltc.governance.service.SrvTaskService;
import com.hungraim.ltc.pojo.entity.SysConfig.ChFndAttach;
import com.hungraim.ltc.pojo.entity.governance.ChSrvPlan;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvProgram;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTask;
import com.hungraim.ltc.pojo.entity.serviceManage.ChSrvTaskDetail;
import com.hungraim.ltc.pojo.vo.governance.*;
import com.hungraim.ltc.pojo.vo.system.AttachUpdateVo;
import com.hungraim.ltc.util.AttachType;
import com.hungraim.ltc.util.DateUtils;
import com.hungraim.ltc.util.Result;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * 服务计划ServiceImpl
 */
@Service
@Slf4j
@AllArgsConstructor
public class SrvTaskServiceImpl implements SrvTaskService {
    private final SrvTaskMapper srvTaskMapper;
    private final ChSrvTaskDetailMapper chSrvTaskDetailMapper;
    private final AttachFeignService attachFeignService;
    private final ChSrvPlanMapper chSrvPlanMapper;
    private final AccountFeignService accountFeignService;
    private final ChSrvProgramMapper chSrvProgramMapper;
    private final ChSrvSendMapper chSrvSendMapper;
    private  final SrvSendService sendService;

    /**
     * 服务计划制定列表
     *
     * @param srvOrganId  护理服务机构id
     * @param disableName 失能人员姓名
     * @param disableTel  失能人员手机号码
     * @param taskStatus  服务计划状态
     */
    @Override
    public Page<SrvTaskVO> getSrvTaskList(Page<SrvTaskVO> page, Long srvOrganId, String disableName, String disableTel, String taskStatus) {
        return srvTaskMapper.getSrvTaskList(page, srvOrganId, disableName, disableTel, taskStatus);
    }

    /**
     * 服务计划查询列表
     *
     * @param organId     护理机构id
     * @param disableName 失能人员姓名
     * @param disableTel  失能人员手机号码
     * @param empName     护理人员姓名
     * @param certifNum   护理人员证件号码
     * @param startTime   服务计划开始时间
     * @param endTime     服务计划结束时间
     * @param status      任务状态
     * @param modeId      服务方式
     */
    @Override
    public Page<SrvTaskDetailVO> getSrvTaskDetailList(Page<SrvTaskDetailVO> page, Integer organId, String disableName, String disableTel, String empName, String certifNum, String srvDate, Date startTime, Date endTime, String status, String modeId) {
        return srvTaskMapper.getSrvTaskDetailList(page, organId, disableName, disableTel, empName, certifNum, srvDate,startTime, endTime, status, modeId);
    }

    /**
     * 查询服务计划制定详情
     *
     * @param programId 服务方案id
     */
    @Override
    public List<SrvTaskInfoVO> getSrvTaskInfoList(String programId) {
        List<SrvTaskInfoVO> srvTaskInfoList = srvTaskMapper.getSrvTaskInfoList(programId);
        //查询服务计划附件
//        try {
//            for (SrvTaskInfoVO vo : srvTaskInfoList) {
//                // 返回对应的附件列表
//                Result<List<ChFndAttach>> listResult = attachFeignService.selectAttachs(Long.parseLong(programId), AttachType.SRV_TASK.getUploadType());
//                if (!listResult.getCode().equals(ResultCode.SUCCESS.getCode())) {
//                    log.info("请求获取附件列表错误：错误码{},错误信息:{}", listResult.getCode(), listResult.getMsg());
//                } else {
//                    vo.setListResult(listResult);
//                }
//            }
//        } catch (Exception e) {
//            log.error(e.getMessage());
//            e.printStackTrace();
//        }
        //具体服务日期
        for (SrvTaskInfoVO vo : srvTaskInfoList) {
            List<ChSrvTaskDetail> taskDetails = chSrvTaskDetailMapper.selectList(new QueryWrapper<ChSrvTaskDetail>().eq("srv_task_id", vo.getSrvTaskId()));
            List<String> srvDateList = new ArrayList<>();
            for (ChSrvTaskDetail detail : taskDetails) {
                Date srvDate = detail.getSrvDate();//服务日期
                String s = DateUtils.dateToStr_yyyy_MM_dd(srvDate);
                srvDateList.add(s);
            }
            vo.setTaskDetails(srvDateList);
        }
        return srvTaskInfoList;
    }

    /**
     * 调用feign api来修改附件数据
     */
    private void updateAttachs(List<ChFndAttach> fileDataList, Long ctrlId) {
        // 新增附件修改逻辑
        AttachUpdateVo attachUpdateVo = new AttachUpdateVo();
        attachUpdateVo.setCtrlId(ctrlId);
        attachUpdateVo.setAttachType(AttachType.SRV_TASK.getUploadType());
        attachUpdateVo.setChFndAttachs(fileDataList);
        attachFeignService.updateAttachs(attachUpdateVo);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void addOrUpdateSrvTask(SrvTaskInfoListVO param) {
        log.info("制定服务信息:" + JSONObject.toJSON(param));
        String programId = param.getProgramId();
        String disabInfoId = param.getDisabInfoId();
        List<SrvTaskInfoVO> taskList = param.getTaskList();
        for (SrvTaskInfoVO srvTaskInfoVO : taskList) {
            String srvTaskId = srvTaskInfoVO.getSrvTaskId();
            List<String> taskDetails = srvTaskInfoVO.getTaskDetails();
            if (StrUtil.isBlank(srvTaskId)) {
                // 添加服务计划
                ChSrvTask chSrvTask = new ChSrvTask();
                chSrvTask.setEffTime(new Date());//创建时间
                chSrvTask.setProgramId(Long.parseLong(programId));//服务方案id
                chSrvTask.setDisabInfoId(Long.parseLong(disabInfoId)); // 失能人员id
                chSrvTask.setSrvEmpId(Long.parseLong(srvTaskInfoVO.getSrvEmpId()));//服务人员id
                chSrvTask.setStatus((short) 1);//状态 未分配：0；已分配：1;已终止2
                chSrvTask.setSrvDate(DateUtils.strToDate(taskDetails.get(0)));
                chSrvTask.setFcd(new Date());
                chSrvTask.setLcd(new Date());
                chSrvTask.setFcu(param.getUserId().longValue());
                chSrvTask.setLcu(param.getUserId().longValue());
                srvTaskMapper.insert(chSrvTask);
                // 服务计划制定方案修改为已生效
                ChSrvProgram chSrvProgram = new ChSrvProgram();
                chSrvProgram.setProgramId(Long.valueOf(programId));
                chSrvProgram.setProgStatus(1);
                chSrvProgramMapper.updateById(chSrvProgram);
                //添加服务计划详情
                Long newSrvTaskId = chSrvTask.getSrvTaskId();
                this.insertTaskDetail(newSrvTaskId, taskDetails);
                saveSrvPlan(programId,null);
                //添加附件
//                this.updateAttachs(srvTaskInfoVO.getListAttach(), chSrvTask.getSrvTaskId());
            } else {
                // 修改服务计划
               // chSrvTaskDetailMapper.delete(new QueryWrapper<ChSrvTaskDetail>().eq("srv_task_id", Long.parseLong(srvTaskId)));//删除原来的计划详情
                /*   if("2".equals(srvTaskInfoVO.getTaskStatus())){
                    UpdateWrapper updateWrapper1 = new UpdateWrapper();
                    updateWrapper1.eq("SRV_TASK_ID",srvTaskId);
                    updateWrapper1.set("STATUS",3);
                    chSrvTaskDetailMapper.update(null,updateWrapper1);
                   }*/
               // this.insertTaskDetail(Long.parseLong(srvTaskId), taskDetails);
                //修改服务计划
                ChSrvTask chSrvTask = new ChSrvTask();
                UpdateWrapper updateWrapper = new UpdateWrapper();
                updateWrapper.eq("SRV_TASK_ID",srvTaskId);
                //状态 未分配：0；已分配：1;已终止2
                updateWrapper.set("STATUS",srvTaskInfoVO.getTaskStatus());
                updateWrapper.set("LCD",new Date());
                updateWrapper.set("LCU",param.getUserId());
                updateWrapper.set("END_REASON",srvTaskInfoVO.getEndReason());

                srvTaskMapper.update(null,updateWrapper);
                //修改附件
//                this.updateAttachs(srvTaskInfoVO.getListAttach(), Long.parseLong(srvTaskId));
            }
        }
    }


    public void saveSrvPlan(String programId,String srvPlanTime){
        List<ChSrvPlan> chSrvPlans = chSrvPlanMapper.queryPlan(programId,srvPlanTime);
        for (ChSrvPlan chSrvPlan : chSrvPlans) {
            chSrvPlan.setCreationTime(new Date());
            chSrvPlanMapper.insert(chSrvPlan);
        }
    }

    // mode_id  4
    @Override
    public void   otherPlaceHandler(String programId){
        accountFeignService.getOtherPlace(programId);
    }

    /**
     * 插入服务计划详情数据
     *
     * @param srvTaskId   服务计划id
     * @param taskDetails 服务时间集合，时间格式yyyy-MM-dd
     */
    private void insertTaskDetail(Long srvTaskId, List<String> taskDetails) {
        for (String taskDetail : taskDetails) {
            ChSrvTaskDetail chSrvTaskDetail = new ChSrvTaskDetail();
            chSrvTaskDetail.setSrvTaskId(srvTaskId);//taskId
            chSrvTaskDetail.setSrvDate(DateUtils.strToDate(taskDetail));//服务开始时间
            chSrvTaskDetail.setStatus((short) 0);//状态 未执行：0；执行中：1；已执行:2;已失效：3已完成4
            chSrvTaskDetail.setFcd(new Date());
            chSrvTaskDetailMapper.insert(chSrvTaskDetail);
        }
    }

    @Override
    public Page<OrganEmpVO> getSrvOrganEmpList(Page<OrganEmpVO> page, String organId, String name, String certiType, String certifNum) {
        return srvTaskMapper.getSrvOrganEmpList(page, organId, name, certiType, certifNum);
    }


    @Override
    public void timeAllocationSrvTask(String dateStr) {
        List<ChSrvProgram> programEndProgram = chSrvProgramMapper.getProgramEndProgram();
        if (CollectionUtil.isNotEmpty(programEndProgram)) {
            for (ChSrvProgram chSrvProgram : programEndProgram) {
                chSrvProgram.setProgramId(chSrvProgram.getProgramId());
                chSrvProgram.setEndReason("方案时间已到期,自动终止");
                chSrvProgram.setProgStatus(2);
                chSrvProgramMapper.updateById(chSrvProgram);
                Long disabInfoId = chSrvProgram.getDisabInfoId();
                Date programEdnDate = chSrvProgram.getProgramEdnDate();
                String programEdnDateStr = DateUtils.dateToStr(programEdnDate);
                int i = chSrvSendMapper.updateSrvSendInfo(disabInfoId, programEdnDateStr);
                if(i >0 ){
                    log.info("服务分配{}终止成功",disabInfoId);
                }
            }
        }
        // 查询需要执行的计划
        List<AllocationSrvTaskVO> chSrvTasks = srvTaskMapper.getAllocationSrvTask();

        // 生成计划任务   CH_SRV_TASK_DETAIL
        for (AllocationSrvTaskVO chSrvTask : chSrvTasks) {
            Long srvTaskId =  chSrvTask.getSrvTaskId();

            // 根据任务id和服务时间查询是否已经生成过计划，已经生成了就不需要再生成了,已退出
            int chSrvTaskDetailCount = chSrvTaskDetailMapper
                    .getSrvTaskDetail(String.valueOf(srvTaskId), dateStr);
            if (chSrvTaskDetailCount > 0) {
                log.info("{},已经有计划任务了",srvTaskId);
                continue;
            }
            ChSrvTaskDetail chSrvTaskDetail = new ChSrvTaskDetail();
            chSrvTaskDetail.setSrvTaskId(srvTaskId);//taskId
            chSrvTaskDetail.setSrvDate(DateUtils.strToDate(dateStr));//服务开始时间
            chSrvTaskDetail.setStatus((short) 0);//状态 未执行：0；执行中：1；已执行:2;已失效：3已完成4
            chSrvTaskDetail.setFcd(new Date());
            chSrvTaskDetailMapper.insert(chSrvTaskDetail);
            // ch_srv_plan
            String programId = chSrvTask.getProgramId();
            saveSrvPlan(programId,dateStr);

        }
    }

    // 补卡
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Result saveApplyCard(SrvPlanInputVO srvPlanInputVO){
        Long taskDetailId = null;
        // 先查询是否有计划，有计划了不再生成计划
        List<ChSrvTaskDetail> chSrvTaskDetailList = chSrvTaskDetailMapper.selectList(new LambdaQueryWrapper<ChSrvTaskDetail>()
                .eq(ChSrvTaskDetail::getSrvTaskId, srvPlanInputVO.getSrvTaskId())
                .eq(ChSrvTaskDetail::getSrvDate, srvPlanInputVO.getSrvPlanTime()));
        Long modeId = srvPlanInputVO.getModeId();
        if (CollectionUtil.isNotEmpty(chSrvTaskDetailList)) {
            ChSrvTaskDetail chSrvTaskDetail = chSrvTaskDetailList.get(0);
            taskDetailId = chSrvTaskDetail.getTaskDetailId();
            if(modeId == 3){
                int status = chSrvTaskDetailList.stream().map(ChSrvTaskDetail::getStatus).findFirst().orElse((short)0);
                if(status == 1 || status == 4) {
                    return Result.failed("手机app已有任务,不可重复生成");
                }
                ChSrvTaskDetail chSrvTaskDetailDTO = new ChSrvTaskDetail();
                chSrvTaskDetailDTO.setTaskDetailId(taskDetailId);
                // 补卡标识
                chSrvTaskDetailDTO.setIsClock((short) 1);
                chSrvTaskDetailDTO.setClockReason(srvPlanInputVO.getClockReason());
                chSrvTaskDetailDTO.setClockOperator(srvPlanInputVO.getAdmin());
                chSrvTaskDetailMapper.updateById(chSrvTaskDetailDTO);
            }
            List<ChSrvPlan> srvPlanList = chSrvPlanMapper.selectList(new LambdaQueryWrapper<ChSrvPlan>()
                    .eq(ChSrvPlan::getDisabInfoId, srvPlanInputVO.getDisabInfoId())
                    .eq(ChSrvPlan::getSrvPlanTime, srvPlanInputVO.getSrvPlanTime())
                    .eq(ChSrvPlan::getModeId, srvPlanInputVO.getModeId()));
            if(CollectionUtil.isNotEmpty(srvPlanList)){
                return Result.failed("任务列表已有计划,不可重复生成");
            }
        }else {
            // 插入计划表
            ChSrvTaskDetail chSrvTaskDetail = new ChSrvTaskDetail();

            // taskId
            chSrvTaskDetail.setSrvTaskId(srvPlanInputVO.getSrvTaskId());
            // 服务开始时间
            chSrvTaskDetail.setSrvDate(srvPlanInputVO.getSrvPlanTime());
            // 状态 未执行：0；执行中：1；已执行:2;已失效：3已完成4
            chSrvTaskDetail.setStatus((short) 0);
            // 补卡标识
            chSrvTaskDetail.setIsClock((short) 1);
            chSrvTaskDetail.setClockReason(srvPlanInputVO.getClockReason());
            chSrvTaskDetail.setClockOperator(srvPlanInputVO.getAdmin());
            chSrvTaskDetail.setFcd(new Date());
            int insert = chSrvTaskDetailMapper.insert(chSrvTaskDetail);
            if(insert == 0){
                return Result.failed("补卡失败，请重新提交");
            }
            taskDetailId = chSrvTaskDetail.getTaskDetailId();
        }
        try {
            log.info("=======SrvTaskServiceImpl.saveApplyCard.taskDetailId:{}",taskDetailId);
            String[] itemIds = srvPlanInputVO.getItemIds();
            List<ItemDTO> itemDTOS = chSrvPlanMapper.queryItem(srvPlanInputVO.getSrvOrganId());

            for (String r : itemIds) {
                ChSrvPlan  chSrvPlan = new ChSrvPlan();
                chSrvPlan.setItemId(Long.valueOf(r));
                if (CollectionUtil.isNotEmpty(itemDTOS)) {
                    String itemName = itemDTOS.stream().filter(s -> s.getItemId().equals(r)).map(ItemDTO::getItemName).findFirst().orElse(null);
                    chSrvPlan.setItemName(itemName);
                }
                chSrvPlan.setSrvOrganId(srvPlanInputVO.getSrvOrganId());
                chSrvPlan.setModeId(srvPlanInputVO.getModeId());
                chSrvPlan.setCreationTime(new Date());
                chSrvPlan.setDisabInfoId(srvPlanInputVO.getDisabInfoId());
                chSrvPlan.setEmpId(srvPlanInputVO.getEmpId());
                chSrvPlan.setSrvPlanTime(srvPlanInputVO.getSrvPlanTime());
                chSrvPlan.setTaskDetailId(taskDetailId);
                chSrvPlan.setIsStatus(Long.valueOf(0));
                int insert1 = chSrvPlanMapper.insert(chSrvPlan);
                if(insert1==0){
                    return Result.failed("补卡失败，请重新提交");
                }
            }
        } catch (Exception e) {
            log.error("=======SrvTaskServiceImpl error {}",e);
            return Result.failed("补卡失败");
        }
        return Result.success();
    }


    @Override
    public Result  queryItemBySrvOrganId(Long srvOrganId){
        List<ItemDTO> itemDTOS = chSrvPlanMapper.queryItem(srvOrganId);
        return Result.success(itemDTOS);
    }
}
